#
#
#
#
#
#
#
#
#
#
# Read me first:
#
# Before you start you should make sure that the working directory is set to the
# directory of this file.
#
# You can find an explanation of this experiment on our website:
# https://analyse-gps.com/experiments/ut-smuggle-experiment/
#
#
#
#
#
#
#
#
#
#

library(psyosphere)

# 00 Check directories ---------------------------------------------------------

if (!file.exists("smuggler1.R")) {stop("Please set the working dir correctly")}

dir.create("smuggler1", showWarnings = FALSE)

dir.create("smuggler1/rdata", showWarnings = FALSE)
dir.create("smuggler1/plots", showWarnings = FALSE)
dir.create("smuggler1/gps_files", showWarnings = FALSE)

# 01 store GPX files in data frame ---------------------------------------------

# Download and unzip files
url <- "https://analyse-gps.com/downloads/smuggle1.zip"
download.file(url = url, destfile = "smuggler1/gps_files.zip")
unzip("smuggler1/gps_files.zip", exdir = "smuggler1/gps_files")
remove(url)

# Save GPS files with participant movement as dataframe
tracks <- psyosphere::dir_get_gpx("smuggler1/gps_files")

# Add additional data from CSV file
tracks <- psyosphere::dir_add_csv(tracks, "smuggler1/gps_files/ids.csv")

save.image("smuggler1/rdata/01.RData")

# 02 remove data outside experiment time ---------------------------------------

# The experiment took place between 13:45 and 14:55. Therefore we select this
# data.

rm(list = ls(all.names = TRUE))
load("smuggler1/rdata/01.RData")

begin <- as.POSIXct("2014-09-04 13:45:00")
end <- as.POSIXct("2014-09-04 14:55:00")

tracks <- tracks[ tracks[,"time"] > begin & tracks[,"time"] < end ,]

remove(begin, end)

save.image("smuggler1/rdata/02.RData")

# 03 Create test sample --------------------------------------------------------

# Uncomment the following code to compress the track data. This will speed up
# all calculations that otherwise can take some hours.

rm(list = ls(all.names = TRUE))
load("smuggler1/rdata/02.RData")

tracks <- average_coordinates(tracks, 10, "seconds")

save.image("smuggler1/rdata/03.RData")

# 04 plot all data -------------------------------------------------------------

# Plot all data to get an overview how the data looks like. It will take some
# time before the plot image appears.

rm(list = ls(all.names = TRUE))
load("smuggler1/rdata/03.RData")

plot_overview <- psyosphere::plot_tracks(tracks, line = FALSE)
plot_overview

save.image("smuggler1/rdata/04.RData")

# 05 set start and finish areas ------------------------------------------------

# The participants walked several rounds from start to finish. We are only
# interested in the data between start and finish and want to save them as
# seperate tracks / rounds. First we create the start and finish areas as
# closed polygons.

rm(list = ls(all.names = TRUE))
load("smuggler1/rdata/04.RData")

lon <- c( 6.849975,  6.849627,  6.850001,  6.850350,  6.849975)
lat <- c(52.241745, 52.241100, 52.241004, 52.241649, 52.241745)
lon <- lon - 0.0005
lat <- lat + 0.00012
polygon_start <- data.frame(lon, lat)
remove(lon, lat)

lon <- c(6.851810, 6.851000, 6.851489, 6.852296, 6.851810)
lat <- c(52.241800, 52.240300, 52.240163, 52.241657, 52.241794)
polygon_finish <- data.frame(lon, lat)
remove(lon, lat)

plot_overview <- psyosphere::plot_polygon(polygon_start, plot = plot_overview)
plot_overview <- psyosphere::plot_polygon(polygon_finish, plot = plot_overview)
plot_overview

save.image("smuggler1/rdata/05.RData")

# 06 select tracks between start and finish ------------------------------------

# We select the different rounds of each participant and drop the other data.

rm(list = ls(all.names = TRUE))
load("smuggler1/rdata/05.RData")

tracks <- psyosphere::select_between_polygons(
  tracks, polygon_start, polygon_finish
)

guard_ids <- c("D1","D2","D3","D4","D5")
smuggler_tracks <- tracks[ !tracks[,"team"] %in% guard_ids, ]

plot_rounds <- psyosphere::plot_tracks(smuggler_tracks, line = FALSE, zoom = 17)
plot_rounds <- psyosphere::plot_polygon(polygon_start, plot = plot_rounds)
plot_rounds <- psyosphere::plot_polygon(polygon_finish, plot = plot_rounds)
plot_rounds

remove(smuggler_tracks)

save.image("smuggler1/rdata/06.RData")

# 07 clean-up data -------------------------------------------------------------
rm(list = ls(all.names = TRUE))
load("smuggler1/rdata/06.RData")

# Get descriptive to know what is deleted
descriptives_before <- psyosphere::des_summary(tracks)

# Remove time duplicates
tracks <- psyosphere::average_duplicates(tracks)

# Mark time interval gaps between coordinates
tracks <- psyosphere::t_time_difference(tracks)
tracks <- psyosphere::mark_time_gaps(tracks)

save.image("smuggler1/rdata/07.RData")

# 08 add speed -----------------------------------------------------------------

rm(list = ls(all.names = TRUE))
load("smuggler1/rdata/07.RData")

# Add speed
tracks <- psyosphere::t_speed(tracks)

# Mark impossible speed as gaps
tracks <- psyosphere::mark_speed_gaps(tracks,40)

# Save the gap names for later use
cgaps <- c("time_gap","speed_gap")

save.image("smuggler1/rdata/08.RData")

# 09 add bearing ---------------------------------------------------------------

# This function will add the bearing between each point of a track.

rm(list = ls(all.names = TRUE))
load("smuggler1/rdata/08.RData")

tracks <- psyosphere::t_bearing(tracks)

save.image("smuggler1/rdata/09.RData")

# 10 add distance --------------------------------------------------------------

rm(list = ls(all.names = TRUE))
load("smuggler1/rdata/09.RData")

tracks <- psyosphere::t_distance(tracks)

save.image("smuggler1/rdata/10.RData")

# 11 add distance to finish ----------------------------------------------------

rm(list = ls(all.names = TRUE))
load("smuggler1/rdata/10.RData")

finish <- data.frame(lon = c(6.851810,6.851000), lat = c(52.241800,52.240300))

tracks <- psyosphere::distance_line(tracks,finish, TRUE)

save.image("smuggler1/rdata/11.RData")

# 12 add distance to shortest route from begin ---------------------------------

rm(list = ls(all.names = TRUE))
load("smuggler1/rdata/11.RData")

tracks <- psyosphere::distance_to_direct_line(
  tracks,finish, cname = "deviation"
)

remove(finish)

save.image("smuggler1/rdata/12.RData")

# 13 add average distance to team mates ----------------------------------------

# Be aware this functions can take a lot of time because of the many positions.
# With the following code you can test the calculation first:
#
# tracks_test <- psyosphere::average_coordinates(tracks, 10, "seconds")
# tracks_test <- psyosphere::distance_peers(tracks_test, peer_col = "team")

rm(list = ls(all.names = TRUE))
load("smuggler1/rdata/12.RData")

tracks <- psyosphere::distance_peers(tracks, cpeer = "team")

save.image("smuggler1/rdata/13.RData")

# 14 add distance to border guards ---------------------------------------------

rm(list = ls(all.names = TRUE))
load("smuggler1/rdata/13.RData")

guard_ids <- c("D1","D2","D3","D4","D5")
guard_tracks <- tracks[ tracks[,"team"] %in% guard_ids, ]
tracks <- tracks[ !tracks[,"team"] %in% guard_ids, ]

tracks <- psyosphere::distance_psyo(tracks, guard_tracks)

tracks <- plyr::rename(
  tracks, c(
    "dis_to_D1-01-42.gpx_1_in_m" = "dis_guard1",
    "dis_to_D2-27-27.gpx_1_in_m" = "dis_guard2",
    "dis_to_D3-03-63.gpx_1_in_m" = "dis_guard3",
    "dis_to_D4-04-31.gpx_1_in_m" = "dis_guard4",
    "dis_to_D5-05-55.gpx_1_in_m" = "dis_guard5"
  )
)

save.image("smuggler1/rdata/14.RData")

# 15 mark if border guards are visible -----------------------------------------

rm(list = ls(all.names = TRUE))
load("smuggler1/rdata/14.RData")

polygon_guard_lon <- c(
  6.851234, 6.851129, 6.850950, 6.850821, 6.850699, 6.850582, 6.850477,
  6.850410, 6.850362, 6.850362, 6.850378, 6.850424, 6.850478, 6.850540,
  6.850705, 6.850898, 6.851100, 6.851255, 6.851399, 6.851942, 6.851234
)

polygon_guard_lat <- c(
  52.240456, 52.240448, 52.240478, 52.240531, 52.240601, 52.240666, 52.240772,
  52.240894, 52.241018, 52.241146, 52.241266, 52.241381, 52.241490, 52.241585,
  52.241656, 52.241698, 52.241717, 52.241706, 52.241662, 52.241620, 52.240456
)

polygon_guard <- data.frame(lon = polygon_guard_lon,lat = polygon_guard_lat)
remove(polygon_guard_lon, polygon_guard_lat)

psyosphere::plot_polygon(polygon_guard, plot = plot_rounds)

tracks <- psyosphere::mark_inside_polygon(
  tracks, polygon_guard, cname = "guard_vis"
)

save.image("smuggler1/rdata/15.RData")

# 16 create summary data frame -------------------------------------------------

rm(list = ls(all.names = TRUE))
load("smuggler1/rdata/15.RData")

cgaps <- c("time_gap","speed_gap")
cweight <- "time_difference"

# Descriptives of the whole tracks
summary <- psyosphere::des_summary(tracks, cgaps = cgaps)
summary <- psyosphere::des_first(tracks, "track", cname = "round", des_df = summary)

summary <- psyosphere::des_sd(
  tracks, ctarget = "speed", cweight = cweight, cgaps = cgaps,
  cname = "sd_kmh", des_df = summary
)

summary <- psyosphere::des_mean(
  tracks, ctarget = "average_dis", cweight = cweight,
  cgaps = cgaps, cname = "mean_team_distance", des_df = summary
)

summary <- psyosphere::des_mean(
  tracks, ctarget = "deviation", cweight = cweight,
  cgaps = cgaps, cname = "mean_deviation_route", des_df = summary
)

summary <- psyosphere::des_sd(
  tracks, ctarget = "deviation", cweight = cweight, cgaps = cgaps,
  cname = "sd_deviation_route", des_df = summary
)

# Before hill descriptives

bh_tracks <- tracks[tracks$guard_vis %in% c(0),]

summary <- psyosphere::des_mean(
  bh_tracks, ctarget = "speed", cweight = cweight,
  cgaps = cgaps, cname = "bh_mean_kmh", des_df = summary
)

summary <- psyosphere::des_sd(
  bh_tracks, ctarget = "speed", cweight = cweight, cgaps = cgaps,
  cname = "bh_sd_kmh", des_df = summary
)

summary <- psyosphere::des_mean(
  bh_tracks, ctarget = "average_dis", cweight = cweight,
  cgaps = cgaps, cname = "bh_mean_team_distance", des_df = summary
)

summary <- psyosphere::des_mean(
  bh_tracks, ctarget = "deviation", cweight = cweight,
  cgaps = cgaps, cname = "bh_mean_deviation_route", des_df = summary
)

summary <- psyosphere::des_sd(
  bh_tracks, ctarget = "deviation", cweight = cweight, cgaps = cgaps,
  cname = "bh_sd_deviation_route", des_df = summary
)

# After hill descriptives

ah_tracks <- tracks[tracks$guard_vis %in% c(1),]

summary <- psyosphere::des_mean(
  ah_tracks, ctarget = "speed", cweight = cweight,
  cgaps = cgaps, cname = "ah_mean_kmh", des_df = summary
)

summary <- psyosphere::des_sd(
  ah_tracks, ctarget = "speed", cweight = cweight, cgaps = cgaps,
  cname = "ah_sd_kmh", des_df = summary
)

summary <- psyosphere::des_mean(
  ah_tracks, ctarget = "average_dis", cweight = cweight,
  cgaps = cgaps, cname = "ah_mean_team_distance", des_df = summary
)

summary <- psyosphere::des_mean(
  ah_tracks, ctarget = "deviation", cweight = cweight,
  cgaps = cgaps, cname = "ah_mean_deviation_route", des_df = summary
)

summary <- psyosphere::des_sd(
  ah_tracks, ctarget = "deviation", cweight = cweight, cgaps = cgaps,
  cname = "ah_sd_deviation_route", des_df = summary
)

save.image("smuggler1/rdata/16.RData")

# 17 export to excel -----------------------------------------------------------

rm(list = ls(all.names = TRUE))
load("smuggler1/rdata/16.RData")

library(xlsx)
xlsx::write.xlsx(summary, "smuggler1/smuggler1.xlsx")

save.image("smuggler1/rdata/17.RData")

# 18 set colors per track ------------------------------------------------------

rm(list = ls(all.names = TRUE))
load("smuggler1/rdata/17.RData")

# create dir

dir.create("smuggler1/tracks", showWarnings = FALSE)

# download and unzip file with package data

url <- "https://analyse-gps.com/downloads/packages.zip"
download.file(url = url, destfile = "smuggler1/packages.zip")
unzip("smuggler1/packages.zip", exdir = "smuggler1")
remove(url)

load("smuggler1/packages.RData")

# Smuggler tracks

ah_tracks[,"hist_track_id"] <- paste0("S1_",ah_tracks[,"id"])
ah_tracks <- merge(ah_tracks, package, by = "hist_track_id", all.x = TRUE)
ah_tracks[,"track_color"] <- ifelse(ah_tracks[,"cocaine"], "#00000099", "#ffffff99")
ah_tracks[,"track_name"] <- as.character(ah_tracks[,"team"])

ah_tracks[,"file_name"] <- paste0(
  "smuggler1/tracks/", ah_tracks[,"hist_track_id"], ".kml"
)

# Guard tracks

guard_tracks[,"track_color"] <- "#00ff0000"
guard_tracks[,"track_name"] <- substr(guard_tracks[,"id"], 1, 2)
guard_tracks[,"track_name"] <- gsub("D", "G", guard_tracks[,"track_name"])

guard_tracks[,"file_name"] <- paste0(
  "smuggler1/tracks/", guard_tracks[,"track_name"], ".kml"
)

save.image("smuggler1/rdata/18.RData")

# 19 plot tracks with colors ---------------------------------------------------

rm(list = ls(all.names = TRUE))
load("smuggler1/rdata/18.RData")

ah_tracks[,"team"] <- as.character(ah_tracks[,"team"])

ah_tracks_sel <- ah_tracks[
  grepl("A[0-9]",ah_tracks[,"team"]) &
    ah_tracks[,"track"] == 1,
]

plot_info <- psyosphere::plot_map( rbind(polygon_start, polygon_finish), zoom = 18)
plot_info <- psyosphere::plot_polygon(polygon_start, plot = plot_info)
plot_info <- psyosphere::plot_polygon(polygon_finish, plot = plot_info)
plot_info <- psyosphere::plot_polygon(polygon_guard, plot = plot_info)

psyosphere::plot_tracks(
  ah_tracks_sel, plot = plot_info, dots = FALSE
)

save.image("smuggler1/rdata/19.RData")

# 20 export tracks with colors -------------------------------------------------

rm(list = ls(all.names = TRUE))
load("smuggler1/rdata/19.RData")

psyosphere::export_kml(ah_tracks)
psyosphere::export_kml(guard_tracks)

save.image("smuggler1/rdata/20.RData")
