
# Print title
cat("\nTesting distance_peers()\n")

# Test when peers don't have matching coordinates ------------------------------

# Get data
data(psyo_rounds2)
tracks <- psyo_rounds2
tracks <- select_test_sample(tracks)

# Calculation
distances <- distance_peers(tracks)

# Check results
if (NROW(distances) != 27) { stop("Wrong number of observations") }
if (NCOL(distances) != 11) { stop("Wrong number of variables") }
test_sum <- sum(distances$average_dis)
if (round(test_sum,3) != round(23.9723950659247,3)) {stop("Wrong test_sum")}

# Test single intersect position with same timestamp ---------------------------

#
#  p1 --------x-------> p2
#             ^
#             |
#             |
#            p3
#
# We have two tracks. From track 1 we have p1 and p2 at a certain time. From
# tracks 2 we have point 3. Point 3 has a time between p1 and p2. We want to
# know what the distance from point 3 is to track 1 at the time of point 3. For
# this we need determine point x.
#

# Create the points
p1 <- data.frame(time = "2016-01-01 01:00:00", lon = 0, lat = 0, id = "1")
p2 <- data.frame(time = "2016-01-01 03:00:00", lon = 2, lat = 0, id = "1")
p3 <- data.frame(time = "2016-01-01 02:00:00", lon = 1, lat = 1, id = "2")
p1$time <- as.POSIXct(p1$time)
p2$time <- as.POSIXct(p2$time)
p3$time <- as.POSIXct(p3$time)

# Combine into a line for illustration on a map
# line <- rbind(p1, p2, p3)
# plot_tracks(line)

# As one could gess x should heve the coordinates lon 0 and lat 1. With the
# following map you see that this is the case.
x <- psyosphere:::timed_destination_point_private(p1, p2, p3, "id")
# x$ele <- NULL
# line <- rbind(line, x)
# plot_tracks(line)

# Check result
if (as.integer(x[1,"lon"]) != 1 || as.integer(x[1,"lat"]) != 0) {
  stop("Intersection point is wrong.")
}

# Test if participants distance to him/her-self is 0 ---------------------------

# Get data of only one participant
data(psyo_rounds2)
self_rounds <- psyo_rounds2
self_rounds <- psyosphere::select_test_sample(psyo_rounds2, 10)
self_rounds <- self_rounds[(
  (self_rounds[,c("id")] %in% c("17.gpx_1", "17.gpx_2", "17.gpx_3"))
),]

# Get distance
self_rounds <- psyosphere::distance_peers(self_rounds, single = TRUE)

# Check if participants distance to him/her-self is 0
total_sum <- sum(self_rounds$dis_to_17.gpx_1_in_m, na.rm = TRUE) +
  sum(self_rounds$dis_to_17.gpx_2_in_m, na.rm = TRUE) +
  sum(self_rounds$dis_to_17.gpx_3_in_m, na.rm = TRUE)
if (total_sum != 0) { stop("Distance sum participant to him/her-self not 0.") }

# Check if participants average distance is 0
sum2 <- as.numeric(sum(self_rounds$average_dis))
if (sum2 != 0) {stop("Average distance participant to him/her-self not 0.")}
if (all(is.na(self_rounds$average_dis))) {stop("All average distances are NA")}

# Test whole calculation of three participants ---------------------------------

# Get data
data(psyo_rounds2)
average_rounds <- psyo_rounds2
average_rounds <- psyosphere::select_test_sample(average_rounds, 10)

# Calculate distances
average_rounds <- psyosphere::distance_peers(average_rounds, single = TRUE)
average_sum <- sum(average_rounds$average_dis)

# Check result
if (NCOL(average_rounds) != 20) { stop( paste0("Not enough columns",NCOL(average_rounds))) }
if (NROW(average_rounds) != 90) { stop("Not rows") }
e <- val_psyo(average_rounds); if (e != "") {stop(e)}
if (round(average_sum,5) != round(319.095514785691,5)) {
  stop("Value not as expected")
}
